@@ -10,23 +10,24 @@ import fenics
 import mshr
 import sys
 import numpy as np
-import ROOT
+import matplotlib.pyplot as plt
 from array import array
 
 
 #Calculate the weighting potential and electric field
 class FenicsCal:
 
-    def __init__(self,my_d,fen_dic):
+    def __init__(self,my_d,fen_dic,det_dic):
         self.p_electric = []
         self.w_p_electric = []
         self.det_model = fen_dic['name']
         self.fl_x=my_d.l_x/fen_dic['xyscale']  
-        self.fl_y=my_d.l_y/fen_dic['xyscale']  
+        self.fl_y=my_d.l_y/fen_dic['xyscale']
         self.fl_z=my_d.l_z
         self.tol = 1e-14
-        m_sensor_box=self.fenics_space(my_d)  
-        self.mesh3D = mshr.generate_mesh(m_sensor_box,fen_dic['mesh'])  
+        self.det_dic=det_dic
+        m_sensor_box=self.fenics_space(my_d)
+        self.mesh3D = mshr.generate_mesh(m_sensor_box,fen_dic['mesh'])
         self.V = fenics.FunctionSpace(self.mesh3D, 'P', 1)
         self.fenics_p_electric(my_d)
         self.fenics_p_w_electric(my_d)
@@ -56,7 +57,7 @@ class FenicsCal:
             m_sensor =  mshr.Box(fenics.Point(0, 0, 0), 
                                  fenics.Point(self.fl_x, self.fl_y, self.fl_z))
         else:
-            print("sensor model is wrrong")
+            print("sensor model is wrong")
             sys.exit()
         return m_sensor            
         
@@ -121,7 +122,24 @@ class FenicsCal:
 
         u = fenics.TrialFunction(self.V)
         v = fenics.TestFunction(self.V)
-        f = fenics.Constant(self.f_value(my_d))
+        if self.det_dic['name']=="lgad3D":
+            if self.det_dic['part']==2:
+                bond = self.det_dic['bond1']
+                doping_avalanche = self.f_value(my_d,self.det_dic['doping1'])
+                doping = self.f_value(my_d,self.det_dic['doping2'])
+                f = fenics.Expression('x[2] < width ? doping1 : doping2', degree=1,width=bond,doping1=doping_avalanche,doping2=doping)
+            elif self.det_dic['part'] == 3:
+                bond1 = self.det_dic['bond1']
+                bond2 = self.det_dic['bond2']
+                doping1 = self.f_value(my_d,self.det_dic['doping1'])
+                doping2 = self.f_value(my_d,self.det_dic['doping2'])
+                doping3 = self.f_value(my_d,self.det_dic['doping3'])
+                f = fenics.Expression('x[2] < bonda ? dopinga : x[2] > bondb ? dopingc : dopingb', degree = 1, 
+                                         bonda = bond1, bondb = bond2, dopinga=doping1, dopingb = doping2, dopingc = doping3)
+            else:
+                print("The structure of lgad is wrong.")
+        else:
+            f = fenics.Constant(self.f_value(my_d))
         a = fenics.dot(fenics.grad(u), fenics.grad(v))*fenics.dx
         L = f*v*fenics.dx
         # Compute solution
@@ -167,8 +185,13 @@ class FenicsCal:
         """
         bc_l = []
         p_ele,n_ele=self.model_para(my_d,model)
+        x_list = []
+        y_list = []
+        radius = my_d.e_tr[0][2]
         for i in range (len(my_d.e_tr)):
             e_i = my_d.e_tr[i]
+            x_list.append(e_i[0])
+            y_list.append(e_i[1])
             str_e = "x[0]>={e_0}-{e_2} && x[0]<={e_0}+"\
                     +"{e_2} && x[1]>={e_1}-{e_2} && "\
                     +"x[1]<={e_1}+{e_2} && x[2]>={e_3} \
@@ -181,6 +204,8 @@ class FenicsCal:
             else:
                 bc = fenics.DirichletBC(self.V, n_ele, elec_p)
             bc_l.append(bc)
+        bc = self.out_column(x_list,y_list,radius,model,my_d)
+        bc_l.append(bc)
         return bc_l
 
     def boundary_definition_2D(self,my_d,model):
@@ -198,7 +223,25 @@ class FenicsCal:
             return abs(x[2])<self.tol or abs(x[2]-self.fl_z)<self.tol
         bc_l = fenics.DirichletBC(self.V, u_D, boundary)
         return bc_l
-    
+
+    def out_column(self,x_list,y_list,radius,model,my_d):
+        x_min = min(x_list)-radius
+        x_max = max(x_list)+radius
+        y_min = min(y_list)-radius
+        y_max = max(y_list)+radius
+        
+        x_center = my_d.e_tr[0][0]
+        y_center = my_d.e_tr[0][1]
+        str_e = "(x[0]-{e_0})*(x[0]-{e_0}) +(x[1]-{e_1})*(x[1]-{e_1}) >= {e_2}*{e_2}"  
+        elec_p = str_e.format(e_0=x_center, e_1=y_center,
+                              e_2=my_d.e_gap)        
+        if model == "Possion":
+            bc = fenics.DirichletBC(self.V, 0.0, elec_p) 
+        elif model == "Laplace":
+            bc = fenics.DirichletBC(self.V, 1.0, elec_p)     
+        return bc    
+
+        
     def model_para(self,my_d,model):
         """
         @description:
@@ -217,7 +260,7 @@ class FenicsCal:
             print("The input fenics solver model is wrong")
         return p_ele,n_ele
 
-    def f_value(self,my_d):
+    def f_value(self,my_d,input_doping=None):
         """
         @description: 
             Cal f_value of Poisson equation
@@ -236,7 +279,10 @@ class FenicsCal:
             print("material is wrong")            
         e0 = 1.60217733e-19
         perm0 = 8.854187817e-12   #F/m
-        f_value = e0*my_d.d_neff*1e6/perm0/perm_mat
+        if self.det_dic['name']=="lgad3D":
+            f_value = e0*input_doping*1e6/perm0/perm_mat
+        else:
+            f_value = e0*my_d.d_neff*1e6/perm0/perm_mat
         return f_value
         
     def get_e_field(self,px,py,pz):
@@ -332,287 +378,4 @@ class FenicsCal:
         return out_range
     
     def __del__(self):
-        pass
-
-
-class FenicsCal2D:
-
-    def __init__(self,det):
-        
-        self.det = det
-
-        # poential & field
-        self.potential_value_2d = []
-    
-        self.electric_field_x_value = [ [] for n in range(self.det.ny+1) ]
-        self.electric_field_y_value = [ [] for n in range(self.det.ny) ]
-
-        self.electric_field_x_position = [ [] for n in range(self.det.ny+1) ]
-        self.electric_field_y_position = [ [] for n in range(self.det.ny) ]
-
-        # weighting poential & weighting field
-        self.weighting_potential_value_2d = []
-    
-        self.weighting_electric_field_x_value = [ [] for n in range(self.det.ny+1) ]
-        self.weighting_electric_field_y_value = [ [] for n in range(self.det.ny) ]
-
-        self.weighting_electric_field_x_position = [ [] for n in range(self.det.ny+1) ]
-        self.weighting_electric_field_y_position = [ [] for n in range(self.det.ny) ]
-
-        self.solve()
-        #self.draw()
-
-    def cal_possion(self):
-        
-        width = self.det.det_width
-        thin = self.det.det_thin
-        
-        nx = self.det.nx
-        ny = self.det.ny
-
-        # Create mesh and function space
-        mesh = fenics.RectangleMesh(fenics.Point(0, 0), fenics.Point(width, thin), nx, ny)
-        V = fenics.FunctionSpace(mesh, "P", 1)
-
-        # Define boundary condition
-        u_D = fenics.Expression('x[1] < tol? det_voltage : 0', degree = 2,tol = 1E-14,det_voltage = self.det.bias_voltage)
-
-        def boundary(x, on_boundary):
-            return abs(x[1])<1E-14 or abs(x[1]-thin)<1E-14
-
-        bc = fenics.DirichletBC(V, u_D, boundary)
-
-        # Define variational problem
-        u = fenics.TrialFunction(V)
-        v = fenics.TestFunction(V)
-
-        f = fenics.Expression(self.det.doping_epr,degree=2)
-        a = fenics.dot(fenics.grad(u), fenics.grad(v))*fenics.dx
-        L = f*v*fenics.dx #+ g*v*ds
-
-        # Compute solution
-        u = fenics.Function(V)
-        fenics.solve(a == L, u, bc)
-
-        potential_value_1d = u.compute_vertex_values()
-        potential_value_2d = np.array(potential_value_1d).reshape(ny+1,nx+1)
-
-        self.potential_value_2d = potential_value_2d
-
-        # print(self.potential_value_2d)
-
-
-    def cal_weighting_possion(self):
-
-        width = self.det.det_width
-        thin = self.det.det_thin
-        
-        nx = self.det.nx
-        ny = self.det.ny
-
-        # Create mesh and function space
-        mesh = fenics.RectangleMesh(fenics.Point(0, 0), fenics.Point(width, thin), nx, ny)
-        V = fenics.FunctionSpace(mesh, "P", 1)
-
-        # Define boundary condition
-        u_D = fenics.Expression('x[1] < tol? det_voltage : 0', degree = 2,tol = 1E-14,det_voltage = self.det.bias_voltage/abs(self.det.bias_voltage))
-
-        def boundary(x, on_boundary):
-            return abs(x[1])<1E-14 or abs(x[1]-thin)<1E-14
-
-        bc = fenics.DirichletBC(V, u_D, boundary)
-
-        # Define variational problem
-        u = fenics.TrialFunction(V)
-        v = fenics.TestFunction(V)
-
-        f = fenics.Constant(0)
-        a = fenics.dot(fenics.grad(u), fenics.grad(v))*fenics.dx
-        L = f*v*fenics.dx #+ g*v*ds
-
-        # Compute solution
-        u = fenics.Function(V)
-        fenics.solve(a == L, u, bc)
-
-        weighting_potential_value_1d = u.compute_vertex_values()
-        weighting_potential_value_2d = np.array(weighting_potential_value_1d).reshape(ny+1,nx+1)
-
-        self.weighting_potential_value_2d = weighting_potential_value_2d
-
-        return weighting_potential_value_2d
-
-    def cal_electric_field(self):
-
-        width = self.det.det_width
-        thin = self.det.det_thin
-        
-        nx = self.det.nx
-        ny = self.det.ny
-
-        x_step = width/nx
-        y_step = thin/ny
-
-        # x direction
-        for j in range(ny+1):
-            for i in range(nx):
-
-                # electric field
-                tmp_xpos = 0.5*x_step*(2*i+1)
-                tmp_xef = (self.potential_value_2d[j][i] - self.potential_value_2d[j][i+1])/x_step
-                self.electric_field_x_position[j].append(tmp_xpos)
-                self.electric_field_x_value[j].append(tmp_xef)
-
-                # weighting field
-                tmp_wxpos = 0.5*x_step*(2*i+1)
-                tmp_xwef = (self.weighting_potential_value_2d[j][i] - self.weighting_potential_value_2d[j][i+1])/x_step
-                self.weighting_electric_field_x_position[j].append(tmp_wxpos)
-                self.weighting_electric_field_x_value[j].append(tmp_xwef)
-
-
-        # y direction 
-        for j in range(ny):
-            for i in range(nx+1):
-
-                # electric field
-                tmp_ypos = 0.5*y_step*(2*j+1)
-                tmp_yef = (self.potential_value_2d[j][i]- self.potential_value_2d[j+1][i])/y_step
-                self.electric_field_y_position[j].append(tmp_ypos)
-                self.electric_field_y_value[j].append(tmp_yef)
-
-                # weighting field
-                tmp_wypos = 0.5*y_step*(2*j+1)
-                tmp_ywef = (self.weighting_potential_value_2d[j][i] - self.weighting_potential_value_2d[j+1][i])/y_step
-                self.weighting_electric_field_y_position[j].append(tmp_wypos)
-                self.weighting_electric_field_y_value[j].append(tmp_ywef)     
-
-
-
-    def cal_point_field(self,px_point,py_point,input_value):
-
-        width = self.det.det_width
-        thin = self.det.det_thin
-        
-        nx = self.det.nx
-        ny = self.det.ny
-
-        x_step = width/nx
-        y_step = thin/ny
-
-        #Interpolation method 
-        rex_value=px_point%x_step
-        nx_value=int(px_point/x_step)
-        rey_value=py_point%y_step
-        ny_value=int(py_point/y_step)
-
-        if(rex_value>x_step/2):
-            e_v_x1=rex_value-x_step/2
-            nx1_v=nx_value
-            nx2_v=nx_value+1
-        else:
-            e_v_x1=rex_value+x_step/2
-            e_v_x2=x_step-e_v_x1
-            nx1_v=nx_value-1
-            nx2_v=nx_value
-
-        if(rey_value>y_step/2):
-            e_v_y1=rey_value-y_step/2
-            ny1_v=ny_value
-            ny2_v=ny_value+1
-        else:
-            e_v_y1=rey_value+y_step/2
-            e_v_y2=y_step-e_v_y1
-            ny1_v=ny_value-1
-            ny2_v=ny_value
-
-        if (nx_value<=0):
-            r_u=0
-            nx1_v=nx2_v
-        elif (nx_value>=nx-1):
-            r_u=0
-            nx2_v=nx1_v
-        else:
-            r_u=e_v_x1/x_step
-
-        if (ny_value<=0):
-            r_t=0
-            ny1_v=ny2_v
-        elif (ny_value>=ny-1):
-            r_t=0
-            ny2_v=ny1_v
-        else:
-            r_t=e_v_y1/y_step
-
-        value_11=input_value[ny1_v][nx1_v]
-        value_21=input_value[ny1_v][nx2_v]
-        value_12=input_value[ny1_v][nx1_v]
-        value_22=input_value[ny1_v][nx2_v]
-        out_field=0.0
-        out_field=(1-r_u)*(1-r_t)*value_11
-        out_field+=r_u*(1-r_t)*value_21
-        out_field+=r_u*r_t*value_22
-        out_field+=(1-r_u)*r_t*value_12
-
-        return out_field        
-
-
-    def solve(self):
-
-        self.cal_possion()
-        self.cal_weighting_possion()
-        self.cal_electric_field()
-
-    def draw(self):
-
-        cutline = int(self.det.nx/2.0)
-
-        # plot electric field at x = middle
-        ep = array( 'd' )
-        ev = array( 'd' )
-
-        for i in range(self.det.ny):
-
-            ep.append(self.electric_field_y_position[i][cutline])
-
-            efx = self.cal_point_field(self.det.det_width/2.0, self.electric_field_y_position[i][cutline], self.electric_field_x_value)
-            efy = self.cal_point_field(self.det.det_width/2.0, self.electric_field_y_position[i][cutline], self.electric_field_y_value)
-            ef = np.array([efx,efy])
-
-            ef_value = np.linalg.norm(ef)#*1e4 #V/cm
-
-            ev.append(ef_value)
-
-        # print(ep)
-        # print(ev)
-
-        g_e = ROOT.TGraph(self.det.ny,ep,ev)
-
-        g_e.SetLineColor(600)
-        g_e.SetLineWidth(4)
-        g_e.SetTitle( 'Electric Field at Cut Line' )
-        g_e.GetXaxis().SetTitle( 'Depth [um]' )
-        g_e.GetXaxis().SetRangeUser(0,self.det.det_thin)
-        g_e.GetYaxis().SetTitle( 'E [V/um]' )
-
-        g_e.GetXaxis().CenterTitle()
-        g_e.GetXaxis().SetTitleOffset(1.8)
-        g_e.GetXaxis().SetTitleSize(0.05)
-        g_e.GetXaxis().SetLabelSize(0.05)
-        #g_e.GetXaxis().SetNdivisions(505)
-
-        g_e.GetYaxis().CenterTitle()
-        g_e.GetYaxis().SetTitleOffset(1.8)
-        g_e.GetYaxis().SetTitleSize(0.05)
-        g_e.GetYaxis().SetLabelSize(0.05)
-        #g_e.GetYaxis().SetNdivisions(505)
-       
-        c = ROOT.TCanvas( 'c', 'c',500, 500 )
-        c.SetGrid()
-        c.SetLeftMargin(0.18)
-        c.SetBottomMargin(0.18)
-        # c.Divide(1,2)
-
-        c.cd()
-        g_e.Draw()
-        c.Modified()
-        c.Update()
-        c.SaveAs("./fig/%s_%s_electricfield_%dV.pdf"%(self.det.mat_name,self.det.det_model,self.det.bias_voltage))
+        pass
\ No newline at end of file
